<?php

//Funzione necessaria per assicurarsi che la pagina sia attiva dell'inverter sia attiva.
function get_http_response_code($url) {
    $headers = get_headers($url);
    return substr($headers[0], 9, 3);
}

//ip dell'inverter e pagina che fornisce la potenza utile
$ip_fronius = '192.168.178.51';
$url = "http://$ip_fronius/solar_api/v1/GetMeterRealtimeData.cgi?Scope=System";


// **********************
//VARIABILI DA MODIFICARE
// **********************
$frequenza=6;			// intervallo in secondi fra le letture(minimo 6 sec)
$minimo=10; 			// potenza in W da non utilizzare - se impostato negativo preleva dalla rete
						// serve a tenere conto delle tolleranze del variatore di potenza, della resistenza e del contatore.
$pwm_ID=1; 				// Porta GPIO utilizzata per comando PWM (numerazione pin Wiringpi 1= GPIO 18)
$tipo_reg=1;			// Tipo di regolazione: 1 a treni di SEMIONDE / 2 a treni di SINUSOIDI COMPLETE (consigliato 1)
$pwm_range=1600;		// Range PWM : valore consigliato 1600
$pwm_clock=2400;		// Clock PWM:  valore consigliato 2400
						// 		Treni di SEMIONDE					  Treni di SINUSOIDI COMPLETE
						// ______________________________			______________________________
						//	range	|	clock	| step	|			|	range	|	clock	| step	|
						//	4000	|	2400	|	50	|			|	4000	|	3840	|	40	|
						//	2000	|	2400	|	25	|			|	4000	|	2400	|	25	|
						//	1600	|	2400	|	20	|			|	3200	|	2400	|	20	|
						//	800		|	2400	|	10	|			|	1600	|	2400	|	10	|
						//	400		|	2400	|	5	|			|	800		|	2400	|	5	|
						//	160		|	2400	|	2	|			|	320		|	2400	|	2	|
						// ______________________________			______________________________

$resistenza=3000;

// *******************************************
// DA QUI IN POI NON SONO NECESSARIE MODIFICHE
// *******************************************
$controllo=0;								// variabile di controllo x fine giornata
$delta=0;									// delta iniziale
$duty=0;									// valore iniziale da assegnare al duty cycle PWM
$pwm_freq=19200000/($pwm_range*$pwm_clock); // frequenza PWM (Hz)
$pwm_period=1/$pwm_freq;					// periodo PWM (s)
$pwm_pulse=$pwm_period/$pwm_range;			// singolo impulso PWM (s)
$pwm_duty_step=(0.01*$tipo_reg)/$pwm_pulse;	// minimo step duty cycle PWM
$pwm_step=(100/$tipo_reg)/$pwm_freq;		// N. di step (intervalli di regolazione possibili)
$step=$resistenza/$pwm_step;				// step minimo del carico regolabile (W)

//	DECOMMENTARE PER DEBUG

echo "Frequenza PWM: \t$pwm_freq Hz\n";
echo "Periodo PWM : \t$pwm_period s\n";
echo "Impulso PWM : \t$pwm_pulse s\n";
echo "Duty step : \t$pwm_duty_step \n";
echo "N. Step : \t$pwm_step \n";
echo "Reg. minima : \t$step W\n";


//configura la porta GPIO per il PWM

system ("gpio mode ".$pwm_ID." PWM"); 			# imposta la porta GPIO in HW PWM Mode
system ("gpio pwm-ms");							# imposta il modo PWM Mark:Space
system ("gpio pwmr ".$pwm_range);				# imposta il range PWM
system ("gpio clock ".$pwm_ID." ".$pwm_clock);	# imposta il clock PWM
system ("gpio pwm ".$pwm_ID." 0");				# imposta a 0 la porta GPIO PWM (resistenza OFF)

// INIZIO CICLO DI CONTROLLO
while ($controllo < 20){

	$today = date("d.m.Y-H:i:s");	// Data nel formato DD.MM.YYYY-HH:min:sec
	$ORA = date("H");
	//controllo che l'inverter sia raggiungibile e, nel caso non lo fosse incremento il numero di tentativi..
	if(get_http_response_code($url) != "200"){
	    echo("Inverter non trovato, controlla l'indirizzo ip..
			\nTentativi rimanenti: ".(20-$controllo)."
			\nNuovo tentativo tra ".$frequenza." secondi..");
			$controllo++;
			system ("gpio pwm ".$pwm_ID." 0");
			sleep($frequenza);
	}
	else{
		$content = file_get_contents($url);
		$json = json_decode($content, true);
		//controllo che lo SmartMeter sia disponibile e correttamente installato e, nel caso non lo fosse incremento il numero di tentativi..
		if(!isset($json['Body']['Data'][0]['PowerReal_P_Sum'])){
			echo("Contolla la presenza dello SmartMeter..
			\nTentativi rimanenti: ".(20-$controllo)."
			\nNuovo tentativo tra ".$frequenza." secondi..");
			$controllo++;
			system ("gpio pwm ".$pwm_ID." 0");
			sleep($frequenza);
		}
		else{
			//prelevo la potenza dallo SmartMeter, che può essere negativa (immetto nella rete) o positiva (prelevo dalla rete)
			$potenza = -$json['Body']['Data'][0]['PowerReal_P_Sum']; //inverto la potenza in modo da avere valori positivi nel caso di immissioni in rete 
			//se la potenza è negativa e mi trovo dopo le 16:00 incremento il controllo
			if ($potenza<0 and $ORA > 19){$controllo++;}
			$bilancio=$potenza-$minimo+$delta;


		  //se il bilancio è maggiore degli step incrementali della resistenza attivo il pwm al valore corretto,
			//altrimenti spengo il pwm;
			if ($bilancio >= $step){
					$duty=floor($bilancio/$step)*$pwm_duty_step;
					if ($duty >=$pwm_range) $duty=$pwm_range;
					$delta = $duty/$pwm_duty_step*$step;
					system ("gpio pwm ".$pwm_ID." ".$duty);
					sleep($frequenza);
				} else {
					$delta = 0;
					system ("gpio pwm ".$pwm_ID." 0");
					sleep($frequenza);
			}
		}
	}

}
// FINE GIORNATA - CHIUSURA SCRIPT
//echo "Fine giornata: $TODAY\n";
system ("gpio pwm ".$pwm_ID." 0");
exit;

?>
